@@ -20,7 +20,7 @@
 # author Salvo "LtWorf" Tomaselli <tiposchi@tiscali.it>
 
 from relational import relation, parser, optimizer
-
+from relational_readline.termcolor import colored
 import os
 
 print relation
@@ -139,20 +139,20 @@ def run_exec_test(testname):
                 fields_ok = fields_ok and glob[i]==exp_result[i]
                 
             if fields_ok:
-                print "\033[32;1mTest passed\033[0m"
+                print colored('Test passed','green')
                 return True
     except:
         pass
-    print "\033[31;1mERROR\033[0m"
-    print "\033[31;1m=====================================\033[0m"
+    print colored('ERROR','red')
+    print colored('=====================================','red')
     print "Expected %s" % exp_result
     #print "Got %s" % result
-    print "\033[31;1m=====================================\033[0m"
+    print colored('=====================================','red')
     return False
 
 def run_py_test(testname):
     '''Runs a python test, which evaluates expressions directly rather than queries'''
-    print "Running expression python test: \033[35;1m%s\033[0m" % testname
+    print "Running expression python test: " + colored (testname,'magenta')
     
     try:
     
@@ -163,15 +163,16 @@ def run_py_test(testname):
         exp_result=eval(expr,rels) #Evaluating the expression
     
         if result==exp_result:
-            print "\033[32;1mTest passed\033[0m"
+            print colored('Test passed','green')
             return True
     except:
         pass
-    print "\033[31;1mERROR\033[0m"
-    print "\033[31;1m=====================================\033[0m"
+   
+    print colored('ERROR','red')
+    print colored('=====================================','red')
     print "Expected %s" % exp_result
     print "Got %s" % result
-    print "\033[31;1m=====================================\033[0m"
+    print colored('=====================================','red')
     return False
 
 def run_test(testname):
@@ -181,7 +182,7 @@ def run_test(testname):
     testname.result
     The query will be executed both unoptimized and
     optimized'''
-    print "Running test: \033[35;1m%s\033[0m" % testname
+    print "Running test: "+ colored(testname,'magenta')
     
     query=None;expr=None;o_query=None;o_expr=None
     result_rel=None
@@ -191,11 +192,15 @@ def run_test(testname):
     
     try:
         result_rel=relation.relation('%s%s.result' % (tests_path,testname))
-        
+       
         query=readfile('%s%s.query' % (tests_path,testname)).strip()
+        query= unicode(query,'utf-8')
+        
+        expr=parser.parse(query)#Converting expression to python code
+     
         o_query=optimizer.optimize_all(query,rels)
     
-        expr=parser.parse(query)#Converting expression to python code
+        
         result=eval(expr,rels) #Evaluating the expression
     
         o_expr=parser.parse(o_query)#Converting expression to python code
@@ -203,15 +208,15 @@ def run_test(testname):
     
     
         if (o_result==result_rel) and (result==result_rel):
-            print "\033[32;1mTest passed\033[0m"
+            print colored('Test passed','green')
             return True
     except Exception as inst:
-        print inst
+        print inst,"------"
         pass
-    print "\033[31;1mERROR\033[0m"
+    print colored('ERROR','red')
     print "Query: %s -> %s" % (query,expr)
     print "Optimized query: %s -> %s" % (o_query,o_expr)
-    print "\033[31;1m=====================================\033[0m"
+    print colored('=====================================','red')
     print "\033[33;1mExpected result\033[0m"
     print result_rel
     print "\033[33;1mResult\033[0m"
@@ -220,7 +225,7 @@ def run_test(testname):
     print o_result
     print "\033[33;1moptimized result match\033[0m", result_rel==o_result
     print "\033[33;1mresult match          \033[0m", result==result_rel
-    print "\033[31;1m=====================================\033[0m"
+    print colored('=====================================','red')
     return False
 
     
