@@ -24,7 +24,7 @@ import os
 import os.path
 import getopt
 from relational import relation, parser
-version="1.0"
+version="1.1"
 
 
 def printver(exit=True):
@@ -81,17 +81,34 @@ if __name__ == "__main__":
             x11=False
     
     if x11:
-        import sip
-        from PyQt4 import QtCore, QtGui
+        
+        pyqt=True
         
         try:
-            from relational_gui import maingui,guihandler, about, surveyForm
+            from PyQt4 import QtGui
         except:
-            print >> sys.stderr, "Module relational_gui is missing.\nPlease install relational package."
-            sys.exit(3)
+            print >> sys.stderr, "PyQt seems to be missing, trying to use Pyside"
+            from PySide import QtCore, QtGui
+            pyqt=False
+        
+        
+        if pyqt:
+            try:
+                from relational_gui import maingui,guihandler, about, surveyForm
+            except:
+                print >> sys.stderr, "Module relational_gui is missing.\nPlease install relational package."
+                sys.exit(3)
+        else:
+            try:
+                from relational_pyside import maingui,guihandler, about, surveyForm
+            except:
+                print >> sys.stderr, "Module relational_pyside is missing.\nPlease install relational package."
+                sys.exit(3)
+        
             
         about.version=version
         surveyForm.version=version
+        guihandler.version=version
 
         app = QtGui.QApplication(sys.argv)
         
@@ -100,7 +117,6 @@ if __name__ == "__main__":
     
         #if os.name=='nt':
         Form.setFont(QtGui.QFont("Dejavu Sans Bold"))
-    
         
         ui.setupUi(Form)
     
@@ -124,5 +140,6 @@ if __name__ == "__main__":
         except:
             print >> sys.stderr, "Module relational_readline is missing.\nPlease install relational-cli package."
             sys.exit(3)
+        relational_readline.linegui.version=version
         relational_readline.linegui.main(files)
 
